@@ -12,7 +12,7 @@ sub options {
 
 sub parse_arg {
     my ($self, @arg) = @_;
-    @arg = ('') if $#arg < 0;
+    return if $#arg < 0;
 
     return $self->arg_co_maybe (@arg);
 }
@@ -20,25 +20,26 @@ sub parse_arg {
 sub run {
     my ($self, $target) = @_;
 
+    my $pool = SVN::Pool->new_default_sub;
     my $fs = $target->{repos}->fs;
     my $root = $fs->revision_root ($fs->youngest_rev);
     my $ann = Algorithm::Annotate->new;
     my @revs;
 
-    my $pool = SVN::Pool->new_default_sub;
     my $hist = $root->node_history ($target->{path});
+    my $spool = SVN::Pool->new_default ($pool);
     $self->{cross} ||= 0;
     while ($hist = $hist->prev($self->{cross})) {
-	$pool->clear;
 	my ($path, $rev) = $hist->location;
 	unshift @revs, [$path, $rev];
+	$spool->clear;
     }
 
     print loc("Annotations for %1 (%2 active revisions):\n", $target->{path}, scalar @revs);
     print '*' x 16;
     print "\n";
     for (@revs) {
-	$pool->clear;
+	$spool->clear;
 	local $/;
 	my ($path, $rev) = @$_;
 	my $content = $fs->revision_root ($rev)->file_contents ($path);
