@@ -1,10 +1,9 @@
 package SVK::Command::Verify;
 use strict;
-our $VERSION = '0.14';
+our $VERSION = $SVK::VERSION;
 
 use base qw( SVK::Command );
 use SVK::XD;
-use SVK::DiffEditor;
 
 sub options {
     ();
